package cmdUser

import (
    "io"
    "os"
    "errors"
    "strings"
    "encoding/csv"
    "nexus-common/caps"
    "nexus-common/params"
    u "nexus-client/utils"
    g "nexus-client/globals"
)

type SetCaps struct {
    Name string
}

func (cmd *SetCaps)GetName() string {
    return cmd.Name
}

func (cmd *SetCaps)GetDesc() []string {
    return []string{
        "Sets a user's capabilities.",
        "Requires USER_SET_CAPS user capability."}
}

func (cmd *SetCaps)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.GetName()+" email [capability ...]")
    u.PrintlnErr("       "+cmd.GetName()+" file.csv")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    usage := `file.csv   2-column CSV file specifying each user's capabilities:
               column1    email
               column2    capabilities (space separated)
List of user capabilities:
`
    caps := caps.GetUserCapsNames()
    i := 0
    count := 0
    for _, v := range caps {
        if (i == 0) {
            usage += "           "
        }
        usage += v + " "
        i += 1
        count += 1
        if (i >= 5 && count < len(caps)) {
            usage = u.AppendNewLine(usage)
            i = 0
        }
    }
    u.PrintlnErr(usage)
}

func (cmd *SetCaps)Run(args []string) int {
    argc := len(args)
    if argc < 1 {
        cmd.PrintUsage()
        return 1
    }

    statusCode := 0

    if argc == 1 && !u.IsEmail(args[0]) {
        // Single argument and it's a CSV file

        csvFile := args[0]

        if u.IsEmail(csvFile) {
            cmd.PrintUsage()
            return 1
        }

        file, err := os.Open(csvFile)
        if err != nil {
            u.PrintlnErr("Error: "+err.Error())
            return 1
        }
        defer file.Close()
        reader := csv.NewReader(file)
        reader.Comma = g.CsvFieldSeparator    // specify the column separator
        reader.Comment = '#'  // lines starting with this symbol are to be ignored
        line := 0
        for {
            line += 1
            columns, err := reader.Read()
            if err == io.EOF {
                break
            }
            if err != nil {
                u.PrintlnErr("FAILED reading "+err.Error())
                statusCode = 1
                continue
            }

            columnCount := len(columns)
            if columnCount != 2 {
                u.PrintlnErr("FAILED reading record on line ",line,": expecting 2 columns")
                statusCode = 1
                continue
            }

            userCaps := &params.UserSetCaps{ make(map[string]int) }

            capsStr := strings.TrimSpace(columns[1])
            if len(capsStr) > 0 {
                caps := strings.Split(capsStr, " ")
                for _, cap := range caps {
                    userCaps.Caps[cap] = 1
                }
            }

            email := columns[0]
            if !u.IsEmail(email) {
                u.PrintlnErr("FAILED reading record on line ",line,": ",email," is not a valid email")
                statusCode = 1
                continue
            }

            if err := cmd.runRequest(email, userCaps); err != nil {
                u.PrintlnErr(err.Error())
                statusCode = 1
            } else {
                u.Println("Successfully set capabilities for "+email)
            }
        }
    } else {
        // Argument is an email address
        email := args[0]

        userCaps := &params.UserSetCaps{ make(map[string]int) }
        for _, cap := range args[1:] {
            userCaps.Caps[cap] = 1
        }

        if err := cmd.runRequest(email, userCaps); err != nil {
            u.PrintlnErr(err.Error())
            statusCode = 1
        } else {
            u.Println("Successfully set capabilities for "+email)
        }
    }

    return statusCode
}

func (cmd *SetCaps)runRequest(email string, userCaps *params.UserSetCaps) error {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    resp, err := client.R().SetBody(userCaps).Put(host+"/users/"+email+"/caps")
    if err != nil {
        return errors.New("Error: "+err.Error())
    }

    if resp.IsSuccess() {
        return nil
    } else {
        return errors.New("Error: "+resp.Status()+": "+resp.String())
    }
}
